@@ -178,11 +178,7 @@ class Spec(object):
         boilerplate.addAtRisk(self)
         addNoteHeaders(self)
         boilerplate.removeUnwantedBoilerplate(self)
-        shorthands.transformShorthandElements(self)
-        shorthands.transformProductionPlaceholders(self)
-        shorthands.transformMaybePlaceholders(self)
-        shorthands.transformAutolinkShortcuts(self)
-        shorthands.transformProductionGrammars(self)
+        shorthands.run(self)
         inlineTags.processTags(self)
         canonicalizeShortcuts(self)
         addImplicitAlgorithms(self)
@@ -271,7 +267,7 @@ class Spec(object):
                 outputFilename = "-"
             elif self.inputSource.sourceName.endswith(".bs"):
                 outputFilename = self.inputSource.sourceName[0:-3] + ".html"
-            elif self.inputSource.endswith(".src.html"):
+            elif self.inputSource.sourceName.endswith(".src.html"):
                 outputFilename = self.inputSource.sourceName[0:-9] + ".html"
             else:
                 outputFilename = "-"
