@@ -7,7 +7,7 @@ from collections import defaultdict
 import attr
 
 from . import constants
-from .htmlhelpers import *
+from .html import *
 from .messages import *
 
 @attr.s(slots=True)
